#include <errno.h>
#include "stat.h"

main(int argc, char **argv)
{
    struct stat st;
    char *cp, *ctime();

    argc--; argv++;
    for (; argc--; argv++) {
	if (stat (*argv, &st) < 0) {
	    perror ("stattest");
	    continue;
	}
	cp = ctime(&st.st_mtime);
	printf ("%5d %3d %6o %d %6d %15.15s %s\n",
		st.st_ino,
		st.st_blocks,
		st.st_mode,
		st.st_nlink,
		st.st_size,
		&cp[4],
		*argv);
    }
}
