main()
{
unsigned long ful, dul;
unsigned int fui, dui;
unsigned short fus, dus;
float flt;
double dbl;

flt = -2.0;
dbl = -3.0;

ful = (unsigned long)flt;
fui = (unsigned int)flt;
fus = (unsigned short)flt;

dul = (unsigned long)dbl;
dui = (unsigned int)dbl;
dus = (unsigned short)dbl;


printf ("fus[%d/0x%x],fui[%d/0x%x],ful[%d/0x%x]\n",fus,fus,fui,fui,ful,ful);
printf ("dus[%d/0x%x],dui[%d/0x%x],dul[%d/0x%x]\n",dus,dus,dui,dui,dul,dul);
}
