#include <stdio.h>

int setenv(char *name, char *val)
{
    char fullname[128];
    FILE *f;
    int len;

    sprintf(fullname, "ENV:%s", name);
    if (!(f = fopen(fullname, "w"))) return;
    len = fwrite(val, 1, strlen(val), f);
    fclose(f);
    return(0);
}
