#include <libraries/dosextens.h>
#include <functions.h>

int getpid()
{
    return (((struct Process *)FindTask(0))->pr_TaskNum);
}

int getppid()
{
    return (1);		/* We are all children of "init" */
}
