/* Take a new mask and return the "current" one.  We could look through
   all the "open" and "creat" and "mkdir" commands and make sure they look
   at this, but that won't catch everything, so this is a no-op.
*/
int umask(int mask)
{
    static int oldmask = 022;	/* Standard Unix default */
    int i = oldmask;

    oldmask = mask;
    return (i);
}
