#include <stdio.h>

main(int argc, char **argv)
{
    int i, newline = 1;
    char *cp, *scdir();

    argc--; argv++;
    if (argc > 0 && !strcmp(*argv, "-n")) {
	newline = 0;
	argc--;	argv++;
    }
    for (;argc-- > 0; argv++) {
	for (i = 0; cp = scdir(*argv); i++) {
	    if (i) putchar(' ');
	    fputs (cp, stdout);
	}

	if (argc) putchar(' ');
    }
    if (newline) putchar('\n');
    exit(0);
}
