/* We appear to be root on a system without uid's */
int getuid() {return (0);}
int geteuid() {return (0);}
int getgid() {return (0);}
int getegid() {return (0);}


/* File ownership has no meaning in AmigaDOS */
int chown(char *path, int owner, int group)
{
    return (0);
}
